%{

/*
   conf.l -- PIKT lexer for the PIKT.conf file

   $Header: /var/cvs/PIKT/pikt/src/lib/conf.l,v 1.38 2005/01/04 13:49:31 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998,1999,2000,2001,2002,2003,2004,2005 by Robert Osterlund.

   PIKT is a trademark of the University of Chicago.  All Rights Reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   Send problem reports, bouquets and brickbats to:  pikt-bugs@pikt.org
 */

#include "common.h"
#include "conf.tab.h"

char _conflex_[] = "conflex";
char _confparse_[] = "confparse";

#define RETURN(token) conflval.sval = mystrdup(yytext, _conflex_); return token

int syslog_facilities[] = { LOG_LOCAL0,
                            LOG_LOCAL1,
                            LOG_LOCAL2,
                            LOG_LOCAL3,
                            LOG_LOCAL4,
                            LOG_LOCAL5,
                            LOG_LOCAL6,
                            LOG_LOCAL7
                          };

void conferror(const char *);

%}

W		[ \t]
D		[0-9]
L		[A-Za-z]
DL		[0-9A-Za-z]

%option noyywrap

%%

("//"|"#")[^\n]*\n		{ lineno++; }

^uid				{ confparintp = (int *) &authuid;
				  RETURN(_PARINT);
				}
^gid				{ PIKT_conf_gid = TRUE;
				  confparintp = (int *) &authgid;
				  RETURN(_PARINT);
				}
^master				{ confparstrp = &master;
				  RETURN(_PARSTR);
				}
^domain				{ confparstrp = &domain;
				  RETURN(_PARSTR);
				}
^access				{ confparstrp = &accessmode;
				  RETURN(_PARSTR);
				}

^master_address			{ confparstrp = &master_address;
				  RETURN(_PARSTR);
				}
^authenticate_by_master		{ confparintp = &isauth_by_master;
				  RETURN(_PARBLN);
				}
^authenticate_by_master_address { confparintp = &isauth_by_master_address;
				  RETURN(_PARBLN);
				}

^master_ethernet_address	{ confparstrp = &master_ethernet_address;
				  RETURN(_PARSTR);
				}
^authenticate_by_master_ethernet_address {
				  confparintp = &isauth_by_master_ethernet_address;
				  RETURN(_PARBLN);
				}

^data_encryption_type		{ confparintp = &data_encryption_type;
				  RETURN(_PARINT);
				}
^auth_encryption_type		{ confparintp = &auth_encryption_type;
				  RETURN(_PARINT);
				}
^(private_key|pvtkey)		{ confparstrp = &pvtkey;
				  RETURN(_PARSTR);
				}

^all_services			{ confparsrv = ALL_SERVICES;
				  RETURN(_PARSRV);
				}
^kill_piktd_service		{ confparsrv = KILL_PIKTD_SERVICE;
				  RETURN(_PARSRV);
				}
^kill_piktc_svc_service		{ confparsrv = KILL_PIKTC_SVC_SERVICE;
				  RETURN(_PARSRV);
				}
^kill_service			{ confparsrv = KILL_SERVICE;
				  RETURN(_PARSRV);
				}
^restart_piktd_service		{ confparsrv = RESTART_PIKTD_SERVICE;
				  RETURN(_PARSRV);
				}
^restart_piktc_svc_service	{ confparsrv = RESTART_PIKTC_SVC_SERVICE;
				  RETURN(_PARSRV);
				}
^restart_service		{ confparsrv = RESTART_SERVICE;
				  RETURN(_PARSRV);
				}
^call_back			{ confparsrv = CALL_BACK_SERVICE;
				  RETURN(_PARSRV);
				}
^execute_commands_service	{ confparsrv = EXECUTE_COMMANDS_SERVICE;
				  RETURN(_PARSRV);
				}
^execute_(alerts|scripts)_service { confparsrv = EXECUTE_ALERTS_SERVICE;
				  RETURN(_PARSRV);
				}
^execute_service		{ confparsrv = EXECUTE_SERVICE;
				  RETURN(_PARSRV);
				}
^disable_alerts_service		{ confparsrv = DISABLE_ALERTS_SERVICE;
				  RETURN(_PARSRV);
				}
^enable_alerts_service		{ confparsrv = ENABLE_ALERTS_SERVICE;
				  RETURN(_PARSRV);
				}
^show_alerts_status_service	{ confparsrv = SHOW_ALERTS_STATUS_SERVICE;
				  RETURN(_PARSRV);
				}
^install_(alerts|scripts)_service { confparsrv = INSTALL_ALERTS_SERVICE;
				  RETURN(_PARSRV);
				}
^install_programs_service	{ confparsrv = INSTALL_PROGRAMS_SERVICE;
				  RETURN(_PARSRV);
				}
^install_files_service		{ confparsrv = INSTALL_FILES_SERVICE;
				  RETURN(_PARSRV);
				}
^install_objects_service	{ confparsrv = INSTALL_OBJECTS_SERVICE;
				  RETURN(_PARSRV);
				}
^install_service		{ confparsrv = INSTALL_SERVICE;
				  RETURN(_PARSRV);
				}
^fetch_(alerts|scripts)_service	{ confparsrv = FETCH_ALERTS_SERVICE;
				  RETURN(_PARSRV);
				}
^fetch_programs_service		{ confparsrv = FETCH_PROGRAMS_SERVICE;
				  RETURN(_PARSRV);
				}
^fetch_files_service		{ confparsrv = FETCH_FILES_SERVICE;
				  RETURN(_PARSRV);
				}
^fetch_objects_service		{ confparsrv = FETCH_OBJECTS_SERVICE;
				  RETURN(_PARSRV);
				}
^fetch_service			{ confparsrv = FETCH_SERVICE;
				  RETURN(_PARSRV);
				}
^cksum_(alerts|scripts)_service	{ confparsrv = CKSUM_ALERTS_SERVICE;
				  RETURN(_PARSRV);
				}
^cksum_programs_service		{ confparsrv = CKSUM_PROGRAMS_SERVICE;
				  RETURN(_PARSRV);
				}
^cksum_files_service		{ confparsrv = CKSUM_FILES_SERVICE;
				  RETURN(_PARSRV);
				}
^cksum_objects_service		{ confparsrv = CKSUM_OBJECTS_SERVICE;
				  RETURN(_PARSRV);
				}
^cksum_service			{ confparsrv = CKSUM_SERVICE;
				  RETURN(_PARSRV);
				}
^delete_(alerts|scripts)_service { confparsrv = DELETE_ALERTS_SERVICE;
				  RETURN(_PARSRV);
				}
^delete_programs_service	{ confparsrv = DELETE_PROGRAMS_SERVICE;
				  RETURN(_PARSRV);
				}
^delete_files_service		{ confparsrv = DELETE_FILES_SERVICE;
				  RETURN(_PARSRV);
				}
^delete_objects_service		{ confparsrv = DELETE_OBJECTS_SERVICE;
				  RETURN(_PARSRV);
				}
^delete_service			{ confparsrv = DELETE_SERVICE;
				  RETURN(_PARSRV);
				}
^list_(alerts|scripts)_service	{ confparsrv = LIST_ALERTS_SERVICE;
				  RETURN(_PARSRV);
				}
^list_programs_service		{ confparsrv = LIST_PROGRAMS_SERVICE;
				  RETURN(_PARSRV);
				}
^list_files_service		{ confparsrv = LIST_FILES_SERVICE;
				  RETURN(_PARSRV);
				}
^list_objects_service		{ confparsrv = LIST_OBJECTS_SERVICE;
				  RETURN(_PARSRV);
				}
^list_service			{ confparsrv = LIST_SERVICE;
				  RETURN(_PARSRV);
				}

^homdir				{ confparstrp = &homdir;
				  RETURN(_PARSTR);
				}
^bindir				{ confparstrp = &bindir;
				  RETURN(_PARSTR);
				}
^etcdir				{ confparstrp = &etcdir;
				  RETURN(_PARSTR);
				}
^lokdir				{ confparstrp = &lokdir;
				  RETURN(_PARSTR);
				}
^libdir				{ confparstrp = &libdir;
				  RETURN(_PARSTR);
				}
^vardir				{ confparstrp = &vardir;
				  RETURN(_PARSTR);
				}
^altdir				{ confparstrp = &altdir;
				  RETURN(_PARSTR);
				}
^prgdir				{ confparstrp = &prgdir;
				  RETURN(_PARSTR);
				}
^objdir				{ confparstrp = &objdir;
				  RETURN(_PARSTR);
				}
^cfgdir				{ confparstrp = &cfgdir;
				  RETURN(_PARSTR);
				}
^hstdir				{ confparstrp = &hstdir;
				  RETURN(_PARSTR);
				}
^logdir				{ confparstrp = &logdir;
				  RETURN(_PARSTR);
				}

^arpcmd				{ confparstrp = &arpcmd;
				  RETURN(_PARSTR);
				}
^cksumcmd			{ confparstrp = &cksumcmd;
				  RETURN(_PARSTR);
				}
^cpcmd				{ confparstrp = &cpcmd;
				  RETURN(_PARSTR);
				}
^diffcmd			{ confparstrp = &diffcmd;
				  RETURN(_PARSTR);
				}
^echocmd			{ confparstrp = &echocmd;
				  RETURN(_PARSTR);
				}
^egrepcmd			{ confparstrp = &egrepcmd;
				  RETURN(_PARSTR);
				}
^filecmd			{ confparstrp = &filecmd;
				  RETURN(_PARSTR);
				}
^pscmd				{ confparstrp = &pscmd;
				  RETURN(_PARSTR);
				}
^rmcmd				{ confparstrp = &rmcmd;
				  RETURN(_PARSTR);
				}
^nicecmd			{ confparstrp = &nicecmd;
				  RETURN(_PARSTR);
				}
^catcmd				{ confparstrp = &catcmd;
				  RETURN(_PARSTR);
				}
^rpcinfocmd			{ confparstrp = &rpcinfocmd;
				  RETURN(_PARSTR);
				}
^scpcmd				{ confparstrp = &scpcmd;
				  RETURN(_PARSTR);
				}
^sumcmd				{ confparstrp = &sumcmd;
				  RETURN(_PARSTR);
				}
^tailcmd			{ confparstrp = &tailcmd;
				  RETURN(_PARSTR);
				}
^trcmd				{ confparstrp = &trcmd;
				  RETURN(_PARSTR);
				}

^backup_extension		{ confparstrp = &bakext;
				  RETURN(_PARSTR);
				}

^rpc_timeout			{ confparintp = &rpc_timeout;
				  RETURN(_PARINT);
				}
^rpc_retry_timeout		{ confparintp = &rpc_retry_timeout;
				  RETURN(_PARINT);
				}
^rpcinfo_retries		{ confparintp = &rpcinfo_retries;
				  RETURN(_PARINT);
				}
^execcmdout_limit		{ confparintp = &execcmdout_limit;
				  RETURN(_PARINT);
				}
^restart_piktc_svc_wait		{ confparintp = &restart_piktc_svc_wait;
				  RETURN(_PARINT);
				}
^verbose_log			{ confparintp = &isverboselog;
				  RETURN(_PARBLN);
				}
^auto_syslog			{ confparintp = &isautosyslog;
				  RETURN(_PARBLN);
				}
^fork_delay			{ confparintp = &fork_delay;
				  RETURN(_PARINT);
				}
^debug				{ confparintp = &isdebug;
				  RETURN(_PARBLN);
				}
^tcp_only			{ confparintp = &istcponly;
				  RETURN(_PARBLN);
				}

^sort_messages			{ confparintp = &issortmessages;
				  RETURN(_PARBLN);
				}

^syslog_facility		{ confparintp = &syslog_facility;
				  RETURN(_PARINT);
				}

yes|YES|true|TRUE|on|ON		{ conflval.ival = TRUE;
				  return _BLN;
				}
no|NO|false|FALSE|off|OFF	{ conflval.ival = FALSE;
				  return _BLN;
				}

-?{D}+				{ conflval.ival = atoi(yytext);
				  if (confparintp == &syslog_facility) {
					if ((conflval.ival < 0) || (conflval.ival > 7)) {
						logreturn(OUTPUT, LOG_ERR, ERRID_SYSLOG_FACILITY_INVALID, __FILE__, __LINE__, _conflex_, FALSE, "in PIKT.conf, invalid syslog_facility %d", conflval.ival);
						bailout = TRUE;
					} else {
						conflval.ival = syslog_facilities[conflval.ival];
					}
				  }
				  return _INT;
				}

\"[^\"]+\"			{ *(yytext+yyleng-1) = '\0';
				  conflval.sval = mystrdup(yytext+1, _conflex_);
				  return _STR;
				}

[^ \t\n]+			{ RETURN(_STR); }

{W}				{ ; }

\n				{ lineno++; }

%%

/* conf error handler
 */
void
conferror (const char *message)
{

	logreturn(OUTPUT, LOG_ERR, ERRID_CONF_PARSE_ERROR, __FILE__, __LINE__, _confparse_, FALSE, "PIKT.conf, at or near line %d: %s: %s", lineno, message, yytext);
	bailout = TRUE;
	return;
}
